@@ -28,8 +28,17 @@ This module provides all the models needed by the metrics project.
 """
 from __future__ import annotations
 
+import pickle
 from typing import Any
 
+from metrics.core.constants import XP_WARE_NAME, CAMPAIGN_NAME, CAMPAIGN_TIMEOUT, CAMPAIGN_MEMOUT, CAMPAIGN_XP_WARES, \
+    CAMPAIGN_INPUT_SET, CAMPAIGN_EXPERIMENTS, EXPERIMENT_INPUT, EXPERIMENT_XP_WARE, EXPERIMENT_CPU_TIME, INPUT_NAME, \
+    INPUT_SET_NAME, INPUT_SET_INPUTS
+
+"""
+Global campaign attribute constants
+"""
+
 
 class Model:
     """
@@ -51,6 +60,13 @@ class Model:
         """
         return self.__dict__[key]
 
+    def keys(self):
+        """
+
+        @return:
+        """
+        return self.__dict__.keys()
+
 
 class ExperimentWare(Model):
     """
@@ -62,7 +78,7 @@ class ExperimentWare(Model):
         Set the obligatory values directly in the constructor and gives the rest to the parent constructor.
         @param attributes: dictionnary of attribute to add in this model
         """
-        self.name = attributes.pop('name')
+        self.name = attributes.pop(XP_WARE_NAME)
         super().__init__(attributes)
 
 
@@ -76,14 +92,25 @@ class Campaign(Model):
         Set the obligatory values directly in the constructor and gives the rest to the parent constructor.
         @param attributes: dictionnary of attribute to add in this model
         """
-        self.name = attributes.pop('name')
-        self.timeout = attributes.pop('timeout')
-        self.memout = attributes.pop('memout')
-        self.experiment_wares = attributes.pop('experiment_wares')
-        self.input_set = attributes.pop('input_set')
-        self.experiments = attributes.pop('experiments')
+        self.name = attributes.pop(CAMPAIGN_NAME)
+        self.timeout = attributes.pop(CAMPAIGN_TIMEOUT)
+        self.memout = attributes.pop(CAMPAIGN_MEMOUT)
+        self.experiment_wares = attributes.pop(CAMPAIGN_XP_WARES)
+        self.input_set = attributes.pop(CAMPAIGN_INPUT_SET)
+        self.experiments = attributes.pop(CAMPAIGN_EXPERIMENTS)
         super().__init__(attributes)
 
+    def export(self, file=None):
+        if file is None:
+            return pickle.dumps(self, protocol=pickle.HIGHEST_PROTOCOL)
+        return pickle.dump(self, file, protocol=pickle.HIGHEST_PROTOCOL)
+
+    def get_input_set(self):
+        """
+
+        @return:
+        """
+        return self.input_set
 
 class Experiment(Model):
     """
@@ -95,9 +122,9 @@ class Experiment(Model):
         Set the obligatory values directly in the constructor and gives the rest to the parent constructor.
         @param attributes: dictionnary of attribute to add in this model
         """
-        self.input = attributes.pop('input')
-        self.experiment_ware = attributes.pop('experiment_ware')
-        self.cpu_time = attributes.pop('cpu_time')
+        self.input = attributes.pop(EXPERIMENT_INPUT)
+        self.experiment_ware = attributes.pop(EXPERIMENT_XP_WARE)
+        self.cpu_time = attributes.pop(EXPERIMENT_CPU_TIME)
         super().__init__(attributes)
 
 
@@ -109,9 +136,9 @@ class Input(Model):
     def __init__(self, attributes: dict) -> None:
         """
         Set the obligatory values directly in the constructor and gives the rest to the parent constructor.
-        @param attributes: dictionnary of attribute to add in this model
+        @param attributes: dictionary of attribute to add in this model
         """
-        self.path = attributes.pop('path')
+        self.name = attributes.pop(INPUT_NAME)
         super().__init__(attributes)
 
 
@@ -125,8 +152,13 @@ class InputSet(Model):
         Set the obligatory values directly in the constructor and gives the rest to the parent constructor.
         @param attributes: dictionnary of attribute to add in this model
         """
-        self.name = attributes.pop('name')
-        self.inputs = attributes.pop('inputs')
+        self.name = attributes.pop(INPUT_SET_NAME)
+        self.inputs = attributes.pop(INPUT_SET_INPUTS)
         super().__init__(attributes)
 
+    def get_inputs(self):
+        """
 
+        @return:
+        """
+        return self.inputs
\ No newline at end of file
