@@ -29,21 +29,68 @@ This module provides classes for managing the configuration of Scalpel, which
 describes how to read the data collected during a campaign.
 """
 
+
 from __future__ import annotations
 
-from abc import ABC
+from csv import reader as load_csv
+from abc import ABC, ABCMeta, abstractmethod
 from collections import defaultdict
 from fnmatch import fnmatch
 from os import path, walk
-from typing import Dict, Iterable, List, Optional, Union, Any
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, TextIO
 
 from yaml import safe_load as load_yaml
 
-from metrics.scalpel.config.format import CampaignFormat, InputSetFormat
+from metrics.core.constants import *
+from metrics.scalpel.config.datafile import DataFile, create_data_file
+from metrics.scalpel.config.filters import create_filter
+from metrics.scalpel.config.format import CampaignFormat, InputSetFormat, OutputFormat
 from metrics.scalpel.config.inputset import create_input_set_reader
 from metrics.scalpel.listener import CampaignParserListener
-from metrics.scalpel.config.pattern import UserDefinedPattern, compile_named_pattern, compile_regex, \
-    AbstractUserDefinedPattern, UserDefinedPatterns, NullUserDefinedPattern
+from metrics.scalpel.config.pattern import compile_named_pattern, compile_regex, \
+    AbstractUserDefinedPattern, NullUserDefinedPattern, compile_all_named_patterns, \
+    compile_all_regexes
+
+
+class CsvConfiguration:
+
+    def __init__(self, separator: str = ',', quote_char: Optional[str] = None, has_header: bool = True,
+                 title_separator: str = '.'):
+        """
+        Creates a new CsvReader.
+
+        :param separator: The value separator used in the CSV input.
+        :param quote_char: The character used to quote the fields in the
+                           CSV input, if any.
+        :param has_header: Whether the CSV input has a header line.
+        """
+        self._separator = separator
+        self._quote_char = quote_char
+        self._has_header = has_header
+        self._title_separator = title_separator
+
+    def has_header(self):
+        return self._has_header
+
+    def get_separator(self):
+        return self._separator
+
+    def get_quote_char(self):
+        return self._quote_char
+
+    def get_title_separator(self):
+        return self._title_separator
+
+    def create_loader(self, stream: TextIO):
+        """
+        Reads the associated CSV stream line-by-line.
+
+        :return: The lines of the CSV stream.
+        """
+        if self._quote_char is None:
+            return load_csv(stream, delimiter=self._separator)
+
+        return load_csv(stream, delimiter=self._separator, quotechar=self._quote_char)
 
 
 class LogData:
@@ -52,25 +99,25 @@ class LogData:
     an experiment-ware output file.
     """
 
-    def __init__(self, name: str, pattern: UserDefinedPattern) -> None:
+    def __init__(self, names: Union[str, List[str]], pattern: AbstractUserDefinedPattern) -> None:
         """
         Creates a new LogData.
 
-        :param name: The name of the log data.
+        :param names: The name(s) of the log data.
         :param pattern: The pattern identifying the log data.
         """
-        self._name = name
+        self._names = names if isinstance(names, list) else [names]
         self._pattern = pattern
 
-    def get_name(self) -> str:
+    def get_names(self) -> List[str]:
         """
-        Gives the name of this log data.
+        Gives the names of this log data.
 
-        :return: The name of this log data.
+        :return: The names of this log data.
         """
-        return self._name
+        return self._names
 
-    def extract_value_from(self, string: str) -> Optional[str]:
+    def extract_value_from(self, string: str) -> Tuple[str]:
         """
         Extracts the value corresponding to this log data from the given string.
 
@@ -88,20 +135,16 @@ class ScalpelConfiguration:
     relevant values to retrieve from this campaign.
     """
 
-    def __init__(self, fmt: Optional[CampaignFormat], has_header: bool, quote: str, separator: str, main_file: str,
-                 data_files: Optional[List[str]],
+    def __init__(self, fmt: Optional[CampaignFormat], csv_configuration: CsvConfiguration, main_file: str,
+                 data_files: Optional[Dict[str, DataFile]],
                  log_datas: Optional[Dict[str, List[LogData]]],
-                 hierarchy_depth: Optional[int],
-                 experiment_ware_depth: Optional[int],
-                 custom_parser: Optional[str], file_name_meta: FileNameMetaConfiguration) -> None:
+                 custom_parser: Optional[str], file_name_meta: FileNameMetaConfiguration,
+                 is_success) -> None:
         """
         Creates a new ScalpelConfiguration.
 
         :param fmt: The format in which the results of the campaign are stored.
                     If None, the format will be guessed on a best effort basis.
-        :param has_header:
-        :param quote:
-        :param separator:
         :param main_file: The path of the main file containing data about the
                           campaign.
         :param data_files: The names of the files to be considered for each experiment
@@ -113,16 +156,13 @@ class ScalpelConfiguration:
         """
         assert file_name_meta is not None
         self._format = fmt
-        self._has_header = has_header
-        self._quote = quote
-        self._sep = separator
+        self._csv_configuration = csv_configuration
         self._main_file = main_file
         self._data_files = data_files
         self._log_datas = log_datas
         self._custom_parser = custom_parser
-        self._hierarchy_depth = hierarchy_depth
-        self._experiment_ware_depth = experiment_ware_depth
         self._file_name_meta = file_name_meta
+        self._is_success = is_success
 
     def get_main_file(self) -> str:
         """
@@ -144,14 +184,8 @@ class ScalpelConfiguration:
         """
         return self._format
 
-    def has_header(self) -> bool:
-        return self._has_header
-
-    def get_quote_char(self) -> str:
-        return self._quote
-
-    def get_separator(self) -> str:
-        return self._sep
+    def get_csv_configuration(self):
+        return self._csv_configuration
 
     def is_to_be_parsed(self, file: str) -> bool:
         """
@@ -162,7 +196,7 @@ class ScalpelConfiguration:
         :return: Whether the file must be parsed.
         """
         if self._log_datas is not None:
-            if file in self._log_datas.keys():
+            if any(fnmatch(file, data_file) for data_file in self._log_datas.keys()):
                 return True
 
         if self._data_files is not None:
@@ -192,19 +226,10 @@ class ScalpelConfiguration:
         """
         if None in self._log_datas:
             return self._log_datas[None]
-        return self._log_datas[filename]
-
-    def get_hierarchy_depth(self):
-        depth = self._hierarchy_depth
-        if depth is None:
-            return 1
-        return int(depth)
-
-    def get_experiment_ware_depth(self):
-        depth = self._experiment_ware_depth
-        if depth is None:
-            return None
-        return int(depth)
+        for k, v in self._log_datas.items():
+            if fnmatch(filename, k):
+                return v
+        return []
 
     def get_custom_parser(self) -> Optional[str]:
         """
@@ -217,6 +242,25 @@ class ScalpelConfiguration:
     def get_file_name_meta(self) -> FileNameMetaConfiguration:
         return self._file_name_meta
 
+    def get_is_success(self):
+        return self._is_success
+
+    def get_output_format(self, file: str) -> Tuple[OutputFormat, Any]:
+        fmt = OutputFormat.guess_format(file)
+        if fmt == OutputFormat.CSV:
+            return fmt, CsvConfiguration()
+        if fmt == OutputFormat.CSV2:
+            return fmt, CsvConfiguration(';')
+        if fmt == OutputFormat.TSV:
+            return fmt, CsvConfiguration('\t')
+        return fmt, None
+
+    def get_data_file(self, name):
+        for file, data_file in self._data_files.items():
+            if fnmatch(name, file):
+                return data_file
+        return None
+
 
 class ConfigurationIterator:
     """
@@ -375,7 +419,7 @@ class RawDataConfiguration(ConfigurationIterator, ABC):
     The MappingConfiguration defines the property of the raw data configuration.
     """
 
-    def get_name(self) -> str:
+    def get_name(self) -> Union[str, List[str]]:
         """
         Gives the name of the current raw data.
 
@@ -407,7 +451,7 @@ class RawDataConfiguration(ConfigurationIterator, ABC):
         """
         raise NotImplementedError('Method "get_regex_pattern()" is abstract!')
 
-    def get_regex_group(self) -> Optional[int]:
+    def get_regex_group(self) -> Optional[Tuple[int]]:
         """
         Gives the group identifying the current raw data in the regular expression.
 
@@ -415,7 +459,7 @@ class RawDataConfiguration(ConfigurationIterator, ABC):
         """
         raise NotImplementedError('Method "get_regex_group()" is abstract!')
 
-    def get_compiled_pattern(self) -> UserDefinedPattern:
+    def get_compiled_pattern(self) -> AbstractUserDefinedPattern:
         """
         Gives the compiled pattern identifying the current raw data.
 
@@ -426,15 +470,20 @@ class RawDataConfiguration(ConfigurationIterator, ABC):
         """
         # First, we look for a named pattern.
         simplified_pattern = self.get_simplified_pattern()
+        groups = self.get_regex_group()
         if simplified_pattern is not None:
-            return compile_named_pattern(simplified_pattern)
+            if groups is None:
+                return compile_named_pattern(simplified_pattern)
+            else:
+                return compile_all_named_patterns(simplified_pattern, *groups)
 
         # There is no look pattern: trying a regular expression.
         regex = self.get_regex_pattern()
-        group = self.get_regex_group()
         if regex is not None:
-            group = 1 if group is None else group
-            return compile_regex(regex, group)
+            if groups is None:
+                return compile_regex(regex)
+            else:
+                return compile_all_regexes(regex, *groups)
 
         # The description of the data is missing!
         raise ValueError('A pattern or regex is missing!')
@@ -492,7 +541,7 @@ class DictionaryRawDataConfiguration(ListConfigurationIterator, RawDataConfigura
     list of dictionaries.
     """
 
-    def get_name(self) -> str:
+    def get_name(self) -> Union[str, List[str]]:
         """
         Gives the name of the current raw data.
 
@@ -524,13 +573,16 @@ class DictionaryRawDataConfiguration(ListConfigurationIterator, RawDataConfigura
         """
         return self.current().get('regex')
 
-    def get_regex_group(self) -> Optional[int]:
+    def get_regex_group(self) -> Optional[Tuple[int]]:
         """
         Gives the group identifying the current raw data in the regular expression.
 
         :return: The group in the regular expression, if any.
         """
-        return self.current().get('group')
+        groups = self.current().get('groups')
+        if groups is not None:
+            return tuple(groups)
+        return None
 
 
 class DictionaryConfiguration:
@@ -562,23 +614,10 @@ class FileNameMetaConfiguration:
         """
         raise NotImplementedError('Method "get_regex_pattern()" is abstract!')
 
-    def get_input_group(self) -> Optional[int]:
-        """
-        Gives the group identifying the input name in the regular expression.
+    def get_groups(self) -> Iterable[Tuple[str, int]]:
+        raise NotImplementedError('Method "get_groups()" is abstract!')
 
-        :return: The group in the regular expression, if any.
-        """
-        raise NotImplementedError('Method "get_regex_group()" is abstract!')
-
-    def get_experiment_ware_group(self) -> Optional[int]:
-        """
-        Gives the group identifying the experiment ware in the regular expression.
-
-        :return: The group in the regular expression, if any.
-        """
-        raise NotImplementedError('Method "get_regex_group()" is abstract!')
-
-    def get_compiled_pattern(self) -> AbstractUserDefinedPattern:
+    def get_log_data(self) -> LogData:
         """
         Gives the compiled pattern identifying the current raw data.
 
@@ -588,24 +627,36 @@ class FileNameMetaConfiguration:
                             regular expression was specified for the raw data.
         """
         # First, we look for a named pattern.
+        names = []
+        groups = []
+        for key, value in self.get_groups():
+            names.append(key)
+            groups.append(value)
+
+        pattern = None
         simplified_pattern = self.get_simplified_pattern()
         if simplified_pattern is not None:
-            return compile_named_pattern(simplified_pattern)
+            pattern = compile_all_named_patterns(simplified_pattern, *groups)
 
-        # There is no look pattern: trying a regular expression.
-        regex = self.get_regex_pattern()
-        experiment_ware_group = self.get_experiment_ware_group()
-        input_group = self.get_input_group()
-        if regex is not None:
-            user_defined_patterns = UserDefinedPatterns()
-            if experiment_ware_group is not None:
-                user_defined_patterns.add(compile_regex(regex, experiment_ware_group))
-            if input_group is not None:
-                user_defined_patterns.add(compile_regex(regex, input_group))
-            return user_defined_patterns
+        if pattern is None:
+            regex = self.get_regex_pattern()
+            if regex is not None:
+                pattern = compile_all_regexes(regex, *groups)
 
         # The description of the data is missing!
-        raise ValueError('A pattern or regex is missing!')
+        if pattern is None:
+            raise ValueError('A pattern or regex is missing!')
+
+        return LogData(names, pattern)
+
+    def extract_from(self, file: str) -> Dict[str, str]:
+        log_data = self.get_log_data()
+        extracted_data = log_data.extract_value_from(file)
+        file_name_data = {}
+        if extracted_data is not None:
+            for name, value in zip(log_data.get_names(), extracted_data):
+                file_name_data[name] = value
+        return file_name_data
 
 
 class EmptyFileNameMetaConfiguration(FileNameMetaConfiguration):
@@ -629,14 +680,14 @@ class EmptyFileNameMetaConfiguration(FileNameMetaConfiguration):
         """
         raise ValueError('Empty raw data configuration!')
 
-    def get_input_group(self) -> Optional[int]:
+    def get_groups(self) -> Iterable[Tuple[str, int]]:
         raise ValueError('Empty raw data configuration!')
 
     def get_experiment_ware_group(self) -> Optional[int]:
         raise ValueError('Empty raw data configuration!')
 
-    def get_compiled_pattern(self) -> AbstractUserDefinedPattern:
-        return NullUserDefinedPattern()
+    def get_log_data(self) -> LogData:
+        return LogData([], NullUserDefinedPattern())
 
 
 class DictionaryFileNameMetaConfiguration(FileNameMetaConfiguration, DictionaryConfiguration):
@@ -647,14 +698,289 @@ class DictionaryFileNameMetaConfiguration(FileNameMetaConfiguration, DictionaryC
     def get_regex_pattern(self) -> Optional[str]:
         return self.get('regex')
 
-    def get_input_group(self) -> Optional[int]:
-        return self.get('input_group')
-
-    def get_experiment_ware_group(self) -> Optional[int]:
-        return self.get('experiment_ware')
+    def get_groups(self) -> Iterable[Tuple[str, int]]:
+        groups = self.get('groups')
+        if groups is None:
+            return []
+        return groups.items()
 
 
-class ScalpelConfigurationBuilder:
+class IScalpelConfigurationBuilder(metaclass=ABCMeta):
+    @abstractmethod
+    def _get_mapping(self):
+        """
+        Gives the mapping configuration object allowing to retrieve the data
+        expected by Scalpel from the experiment files.
+
+        :return: The configuration for the mapping.
+        """
+        pass
+
+    @abstractmethod
+    def read_mapping(self):
+        """
+        Reads the mapping allowing to retrieve the data wanted by Scalpel from
+        the experiment files.
+        """
+        pass
+
+    @abstractmethod
+    def read_metadata(self):
+        """
+        Reads the description of the campaign to parse.
+        """
+        pass
+
+    @abstractmethod
+    def read_setup(self):
+        """
+        Reads the description of the experimental setup.
+        """
+        pass
+
+    @abstractmethod
+    def build(self):
+        """
+        Builds the configuration of Scalpel, as specified by the user.
+
+        :return: The built configuration.
+        """
+        pass
+
+    @abstractmethod
+    def _get_campaign_name(self):
+        """
+        Gives the name of the campaign being considered.
+
+        :return: The name of the campaign.
+        """
+        pass
+
+    @abstractmethod
+    def _get_campaign_date(self):
+        """
+        Gives the date of the campaign being considered.
+
+        :return: The date of the campaign.
+        """
+        pass
+
+    @abstractmethod
+    def _get_os_description(self):
+        """
+        Gives the description of the operating system on which the campaign has
+        been executed.
+
+        :return: The description of the OS.
+        """
+        pass
+
+    @abstractmethod
+    def _get_cpu_description(self):
+        """
+        Gives the description of the CPU of the machine(s) on which the campaign
+        has been executed.
+
+        :return: The description of the CPU.
+        """
+        pass
+
+    @abstractmethod
+    def _get_total_memory(self):
+        """
+        Gives the total amount of memory available on the machine(s) on which the
+        campaign has been executed.
+
+        :return: The total amount of memory.
+        """
+        pass
+
+    @abstractmethod
+    def _get_time_out(self):
+        """
+        Gives the time limit set to the experiments in this campaign.
+
+        :return: The configured time limit.
+        """
+        pass
+
+    @abstractmethod
+    def _get_memory_out(self):
+        """
+        Gives the memory limit set to the experiments in this campaign.
+
+        :return: The set configured limit.
+        """
+        pass
+
+    @abstractmethod
+    def read_experiment_wares(self):
+        """
+        Reads the experiment-wares that are considered in the campaign being
+        parsed by Scalpel.
+        """
+        pass
+
+    @abstractmethod
+    def read_input_set(self):
+        """
+        Reads the input set considered in the campaign being parsed by Scalpel.
+        """
+        pass
+
+    @abstractmethod
+    def read_source(self):
+        """
+        Reads the description of the source from which the campaign is to
+        be parsed.
+        """
+        pass
+
+    @abstractmethod
+    def _get_campaign_path(self):
+        """
+        Gives the path of the file containing all the data about the campaign.
+        This file may be either a regular file or a directory.
+
+        :return: The path to the main file of the campaign.
+        """
+        pass
+
+    @abstractmethod
+    def _get_format(self):
+        """
+        Gives the format of the campaign to parse.
+        If not specified, the format is guessed on a best effort basis.
+
+        :return: The format of the campaign to parse, if any.
+        """
+        pass
+
+    @abstractmethod
+    def _guess_directory_format(self):
+        """
+        Guesses the format of the campaign to parse when stored in a directory.
+
+        :return: The format of the campaign, guessed from the deepness of the
+                 file hierarchy rooted at the main file of this campaign.
+        """
+        pass
+
+    @abstractmethod
+    def _guess_regular_format(self):
+        """
+        Guesses the format of the campaign to parse when stored in a regular file.
+
+        :return: The format of the campaign, guessed from the extension of its
+                 main file, or None if it could not be guessed.
+        """
+        pass
+
+    @abstractmethod
+    def _guess_format(self):
+        """
+        Guesses the format of the campaign to parse.
+
+        :return: The guessed format of the campaign, or None if it could
+                 not be guessed.
+        """
+        pass
+
+    @abstractmethod
+    def read_csv_configuration(self):
+        pass
+
+    @abstractmethod
+    def _has_header(self):
+        """
+        Checks whether the input file to parse has a header.
+
+        :return: If the input file to parse has a header.
+        """
+        pass
+
+    @abstractmethod
+    def _quote_char(self):
+        """
+        :return: Return the quote char
+        """
+        pass
+
+    @abstractmethod
+    def _separator(self):
+        """
+        :return: Return the separator
+        """
+        pass
+
+    @abstractmethod
+    def _title_separator(self):
+        pass
+
+    @abstractmethod
+    def _get_custom_parser(self):
+        """
+        Gives the (completely specified) class of the custom parser to use to parse
+        the campaign, if any.
+
+        :return: The class of the parser to use
+        """
+        pass
+
+    @abstractmethod
+    def _get_data_files(self):
+        """
+        Gives the output files to consider for each experiment, which must be in
+        a format that Scalpel natively recognizes (JSON, CSV, etc.).
+        Such files are only meaningful when the campaign is stored in a "deep"
+        file hierarchy.
+
+        :return: The raw data configuration.
+        """
+        pass
+
+    @abstractmethod
+    def read_data(self):
+        """
+        Reads the files from which Scalpel will extract relevant data for the
+        analysis.
+        Such files are only meaningful when the campaign is stored in a
+        directory.
+        """
+        pass
+
+    @abstractmethod
+    def _get_file_name_meta(self):
+        pass
+
+    @abstractmethod
+    def _get_raw_data(self):
+        """
+        Gives the raw data configuration, which describes how to extract data
+        from the log files of the experiment-wares.
+        Such data is only meaningful when the campaign is stored in a
+        directory.
+
+        :return: The raw data configuration.
+        """
+        pass
+
+    @abstractmethod
+    def _get_is_success(self):
+        pass
+
+    @abstractmethod
+    def _log_data(self, key, value):
+        """
+        Notifies the listener about data that has been read.
+
+        :param key: The key identifying the read data.
+        :param value: The value that has been read.
+        """
+        pass
+
+
+class ScalpelConfigurationBuilder(IScalpelConfigurationBuilder):
     """
     The ScalpelConfigurationBuilder allows to build Scalpel's configuration.
     """
@@ -669,14 +995,11 @@ class ScalpelConfigurationBuilder:
         self._listener = listener
         self._main_file = None
         self._format = None
-        self._hierarchy_depth = None
-        self._experiment_ware_depth = None
+        self._csv_configuration = None
         self._data_files = None
         self._log_datas = defaultdict(list)
         self._custom_parser = None
-        self._header = True
-        self._sep = ','
-        self._quote = None
+        self._is_success = None
         self._file_name_meta = EmptyFileNameMetaConfiguration()
 
     def build(self) -> ScalpelConfiguration:
@@ -693,11 +1016,10 @@ class ScalpelConfigurationBuilder:
         self.read_input_set()
         self.read_source()
         self.read_data()
-        return ScalpelConfiguration(self._format, self._header, self._quote, self._sep, self._main_file,
+        return ScalpelConfiguration(self._format, self._csv_configuration, self._main_file,
                                     self._data_files, self._log_datas,
-                                    self._hierarchy_depth,
-                                    self._experiment_ware_depth,
-                                    self._custom_parser, self._file_name_meta)
+                                    self._custom_parser, self._file_name_meta,
+                                    self._is_success)
 
     def read_mapping(self) -> None:
         """
@@ -724,8 +1046,8 @@ class ScalpelConfigurationBuilder:
         """
         Reads the description of the campaign to parse.
         """
-        self._log_data('name', self._get_campaign_name())
-        self._log_data('date', self._get_campaign_date())
+        self._log_data(CAMPAIGN_NAME, self._get_campaign_name())
+        self._log_data(CAMPAIGN_DATE, self._get_campaign_date())
 
     def _get_campaign_name(self) -> Optional[str]:
         """
@@ -747,11 +1069,11 @@ class ScalpelConfigurationBuilder:
         """
         Reads the description of the experimental setup.
         """
-        self._log_data('os', self._get_os_description())
-        self._log_data('cpu', self._get_cpu_description())
-        self._log_data('memory', self._get_total_memory())
-        self._log_data('timeout', self._get_time_out())
-        self._log_data('memout', self._get_memory_out())
+        self._log_data(CAMPAIGN_OS, self._get_os_description())
+        self._log_data(CAMPAIGN_CPU, self._get_cpu_description())
+        self._log_data(CAMPAIGN_MEMORY, self._get_total_memory())
+        self._log_data(CAMPAIGN_TIMEOUT, self._get_time_out())
+        self._log_data(CAMPAIGN_MEMOUT, self._get_memory_out())
 
     def _get_os_description(self) -> Optional[str]:
         """
@@ -817,12 +1139,12 @@ class ScalpelConfigurationBuilder:
         self._main_file = self._get_campaign_path()
         self._format = self._guess_format()
         self._format = self._get_format()
-        self._header = self._has_header()
-        self._quote = self._quote_char()
-        self._sep = self._separator()
-        self._hierarchy_depth = self._get_hierarchy_depth()
-        self._experiment_ware_depth = self._get_experiment_ware_depth()
         self._custom_parser = self._get_custom_parser()
+        self._is_success = self._get_is_success()
+        self.read_csv_configuration()
+
+    def _get_is_success(self):
+        raise NotImplementedError('Method "_get_is_success()" is abstract!')
 
     def _get_campaign_path(self) -> Iterable[str]:
         """
@@ -864,8 +1186,8 @@ class ScalpelConfigurationBuilder:
         """
         for _, dirs, _ in walk(self._main_file[0]):
             if dirs:
-                return CampaignFormat.DEEP_LOG_DIRECTORY
-        return CampaignFormat.FLAT_LOG_DIRECTORY
+                return CampaignFormat.EXPERIMENT_DIRECTORY
+        return CampaignFormat.SINGLE_EXPERIMENT_LOG_FILE
 
     def _guess_regular_format(self) -> Optional[CampaignFormat]:
         """
@@ -880,6 +1202,9 @@ class ScalpelConfigurationBuilder:
         except ValueError:
             return None
 
+    def read_csv_configuration(self):
+        self._csv_configuration = CsvConfiguration(self._separator(), self._quote_char(), self._has_header(), self._title_separator())
+
     def _has_header(self) -> bool:
         """
         Checks whether the input file to parse has a header.
@@ -900,23 +1225,11 @@ class ScalpelConfigurationBuilder:
         """
         raise NotImplementedError('Method "_separator()" is abstract!')
 
-    def _get_hierarchy_depth(self) -> Optional[int]:
+    def _title_separator(self) -> str:
         """
-        Gives the depth of the hierarchy to explore, when the campaign is
-        in the deep log hierarchy format.
-
-        :return: The depth of the hierarchy, if specified.
+        :return: Return the separator
         """
-        raise NotImplementedError('Method "_get_hierarchy_depth()" is abstract!')
-
-    def _get_experiment_ware_depth(self) -> Optional[int]:
-        """
-        Gives the depth of the directory corresponding to the experiment-ware
-        being executed, when the campaign is in the deep log hierarchy format.
-
-        :return: The depth of the experiment-ware directory, if any.
-        """
-        raise NotImplementedError('Method "_get_experiment_ware_depth()" is abstract!')
+        raise NotImplementedError('Method "_separator()" is abstract!')
 
     def _get_custom_parser(self) -> Optional[str]:
         """
@@ -942,7 +1255,30 @@ class ScalpelConfigurationBuilder:
             pattern = raw_data.get_compiled_pattern()
             self._log_datas[file].append(LogData(name, pattern))
             raw_data.next()
-        self._data_files = self._get_data_files()
+
+        self._data_files = {}
+        for data_file in self._get_data_files():
+            if isinstance(data_file, str):
+                self._data_files[data_file] = create_data_file(data_file)
+            else:
+                name = data_file['name']
+                header = data_file.get('has-header')
+                quote_char = data_file.get('quote-char')
+                separator = data_file.get('separator')
+                title_separator = data_file.get('title-separator')
+                parser = data_file.get('parser')
+                fmt_tmp = data_file.get('format')
+                fmt = None if fmt_tmp is None else OutputFormat.value_of(fmt_tmp)
+                if separator is None:
+                    if fmt == CampaignFormat.CSV2:
+                        separator = ';'
+                    elif fmt == CampaignFormat.TSV:
+                        separator = '\t'
+                    else:
+                        separator = ','
+                csv_config = CsvConfiguration(separator, quote_char, header, title_separator)
+                self._data_files[name] = create_data_file(name, fmt, csv_config, parser)
+
 
     def _get_file_name_meta(self) -> FileNameMetaConfiguration:
         raise NotImplementedError('Method "_get_file_name_meta()" is abstract!')
@@ -958,7 +1294,7 @@ class ScalpelConfigurationBuilder:
         """
         raise NotImplementedError('Method "_get_raw_data()" is abstract!')
 
-    def _get_data_files(self) -> Iterable[str]:
+    def _get_data_files(self) -> Iterable:
         """
         Gives the output files to consider for each experiment, which must be in
         a format that Scalpel natively recognizes (JSON, CSV, etc.).
@@ -1064,14 +1400,20 @@ class DictionaryScalpelConfigurationBuilder(ScalpelConfigurationBuilder):
         Reads the experiment-wares that are considered in the campaign being
         parsed by Scalpel.
         """
-
         experiment_wares = self._dict_config.get('experiment-wares')
-        if experiment_wares is not None:
+        if not experiment_wares:
+            return
+        elif isinstance(experiment_wares[0], str):
             for xp_ware in experiment_wares:
                 self._listener.start_experiment_ware()
-                self._listener.log_data('name', xp_ware)
+                self._listener.log_data(XP_WARE_NAME, xp_ware)
+                self._listener.end_experiment_ware()
+        elif isinstance(experiment_wares[0], dict):
+            for xp_ware in experiment_wares:
+                self._listener.start_experiment_ware()
+                for key, value in xp_ware.items():
+                    self._listener.log_data(key, value)
                 self._listener.end_experiment_ware()
-
 
     def read_input_set(self) -> None:
         """
@@ -1084,21 +1426,12 @@ class DictionaryScalpelConfigurationBuilder(ScalpelConfigurationBuilder):
         self._listener.start_input_set()
         fmt = InputSetFormat.value_of(input_set['type'])
         name = input_set['name']
-        self._listener.log_data('name', name)
-        paths = DictionaryScalpelConfigurationBuilder._as_list(input_set['path-list'])
-        kwargs = {}
-
-        if 'family' in self._get('input-set'):
-            kwargs['family'] = self._get('input-set').get('family')
-
-        if 'input-name' in self._get('input-set'):
-            kwargs['name'] = self._get('input-set').get('input-name')
-
+        self._listener.log_data(INPUT_SET_NAME, name)
+        paths = DictionaryScalpelConfigurationBuilder._as_list(input_set['files'])
+        groups = self._get('input-set').get('file-name-meta')
+        file_name_meta = EmptyFileNameMetaConfiguration() if groups is None else DictionaryFileNameMetaConfiguration(groups)
         extensions = self._get('input-set').get('extensions')
-        if extensions is not None:
-            kwargs['extensions'] = extensions
-
-        create_input_set_reader(fmt, **kwargs)(self._listener, paths)
+        create_input_set_reader(fmt, extensions, file_name_meta)(self._listener, paths)
         self._listener.end_input_set()
 
     def _get_campaign_path(self) -> Iterable[str]:
@@ -1147,25 +1480,26 @@ class DictionaryScalpelConfigurationBuilder(ScalpelConfigurationBuilder):
         :return: Return the separator
         """
         sep = self._get('source').get('separator')
-        return sep if sep is not None else ','
+        if sep is not None:
+            return sep
+        if self._format == CampaignFormat.CSV2:
+            return ';'
+        if self._format == CampaignFormat.TSV:
+            return '\t'
+        return ','
 
-    def _get_hierarchy_depth(self) -> Optional[int]:
-        """
-        Gives the depth of the hierarchy to explore, when the campaign is
-        in the deep log hierarchy format.
+    def _title_separator(self) -> str:
+        sep = self._get('source').get('title-separator')
+        if sep is not None:
+            return sep
+        return '.'
 
-        :return: The depth of the hierarchy, if specified.
-        """
-        return self._get('source').get('hierarchy-depth')
 
-    def _get_experiment_ware_depth(self) -> Optional[int]:
-        """
-        Gives the depth of the directory corresponding to the experiment-ware
-        being executed, when the campaign is in the deep log hierarchy format.
-
-        :return: The depth of the experiment-ware directory, if any.
-        """
-        return self._get('source').get('experiment-ware')
+    def _get_is_success(self):
+        expr = self._get('source').get('is-success')
+        if expr is None:
+            return None
+        return create_filter(expr)
 
     def _get_custom_parser(self) -> Optional[str]:
         """
@@ -1195,7 +1529,7 @@ class DictionaryScalpelConfigurationBuilder(ScalpelConfigurationBuilder):
         data_list = DictionaryScalpelConfigurationBuilder._as_list(data)
         return DictionaryRawDataConfiguration(data_list)
 
-    def _get_data_files(self) -> Iterable[str]:
+    def _get_data_files(self) -> Iterable:
         """
         Gives the output files to consider for each experiment, which must be in
         a format that Scalpel natively recognizes (JSON, CSV, etc.).
